import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/interfaces/auth/auth.dart';

mixin CachedLoginMixin on BaseAuth {
  static const String _cachedLoginKey = 'cachedLogin';
  static const String _splitData = '[,,]';
  static const String _splitValue = '[::]';

  static const String phoneKey = 'phone';
  static const String uidPhoneKey = 'uidPhone';

  Future<Map<String, String>?> loadCachedLogin() async {
    late Map<String, String> map;
    try {
      SharedPreferences pref = await SharedPreferences.getInstance();
      String? data = pref.getString(_cachedLoginKey);
      if (data == null) return null;
      if (data.isEmpty) return null;
      map = {
        for (String e in data.split(_splitData))
          e.split(_splitValue).first: e.split(_splitValue).last,
      };
      if (kDebugMode) {
        print('$runtimeType.loadCachedLogin with data: $map');
      }
      return map;
    } catch (e) {
      handelException(e, subTitle: 'loadCachedLogin');
      return null;
    }
  }

  Future removeCachedLogin() async {
    try {
      if (kDebugMode) {
        print('$runtimeType.removeCachedLogin');
      }
      SharedPreferences pref = await SharedPreferences.getInstance();
      await pref.remove(_cachedLoginKey);
    } catch (e) {
      handelException(e, subTitle: 'removeCachedLogin');
    }
  }

  Future saveCachedLogin({required Map<String, String> map}) async {
    try {
      SharedPreferences pref = await SharedPreferences.getInstance();
      String value = map.entries
          .map((e) => '${e.key}$_splitValue${e.value}')
          .join(_splitData);
      if (kDebugMode) {
        print('$runtimeType.saveCachedLogin with data: $value');
      }
      await pref.setString(_cachedLoginKey, value);
    } catch (e) {
      handelException(e, subTitle: 'saveCachedLogin');
    }
  }
}
