import 'package:flutter/foundation.dart';
import 'package:wowit/common.dart';
import 'package:wowit/interfaces/auth/auth.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/models/base_user_model.dart';
import 'package:wowit/sql_service.dart';

mixin SQLLoginMixin on BaseAuth {
  Future<BaseUserModel?> loginSQL({
    required String phone,
    required String uidPhone,
  }) async {
    if (uidPhone.isEmpty || phone.isEmpty) return null;
    try {
      var response = await SQLService.execute(
        'sp_getLogin',
        params: {'phone': phone, 'uidPhone': uidPhone},
      );
      var data = Methods.getList(response, 'info');
      if (data.isEmpty) return null;
      if (data[0].isEmpty) return null;
      var id = Methods.getInt(data[0], 'ID', defaultValue: 0);
      if (id < 1) return null;
      if (kDebugMode) {
        print('$runtimeType loginSQL with phone: $phone, id: $id');
      }
      Common.uid = id;
      return BaseUserModel(data[0]);
    } catch (e) {
      handelException(e, subTitle: 'loginSQL');
      return null;
    }
  }

  Future logoutSQL() async {
    if (kDebugMode) {
      print('$runtimeType.logoutSQL');
    }
    try {
      Common.uid = -1;
    } catch (e) {
      handelException(e, subTitle: 'logoutSQL');
    }
  }

  Future initAccountSQL({
    required String phone,
    required String uidPhone,
  }) async {
    if (uidPhone.isEmpty || phone.isEmpty) return;
    try {
      await SQLService.createAccountIfNotExists(phone, uidPhone);
      if (kDebugMode) {
        print('$runtimeType.initAccountSQL with phone: $phone, uid: $uidPhone');
      }
      return true;
    } catch (e) {
      handelException(e, subTitle: 'initAccountSQL');
      return false;
    }
  }
}
