import 'dart:ui';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:wowit/methods.dart';

class WowImage extends StatefulWidget {
  const WowImage(
    this.dataImage, {
    super.key,
    this.fieldName,
    this.height,
    this.width,
    this.errorBuilder,
    this.boxFit = BoxFit.cover,
    this.loaddingBuilder,
    this.scale = 1.0,
  }) : _url = '', _borderRadius = null;

  const WowImage.url(
    String url, {
    super.key,
    this.fieldName,
    this.height,
    this.width,
    this.errorBuilder,
    this.boxFit = BoxFit.cover,
    this.loaddingBuilder,
    this.scale = 1.0,
  })  : dataImage = const {},
        _url = url,_borderRadius = null;

  const WowImage.border(
    this.dataImage, {
    super.key,
    this.fieldName,
    this.height,
    this.width,
    this.errorBuilder,
    this.boxFit = BoxFit.cover,
    this.loaddingBuilder,
    this.scale = 1.0,
    BorderRadius? borderRadius
  }) : _url = '',_borderRadius = borderRadius;    

  /// Data hình ảnh
  final Map<String, Object?> dataImage;

  final String _url;

  /// field name hình ảnh [fieldName]!= ['hinhAnh']
  final String? fieldName;

  final BorderRadius? _borderRadius;

  final double? width;
  final double? height;

  final Widget? errorBuilder;

  final BoxFit boxFit;

  final Widget? loaddingBuilder;

  final double scale;

  @override
  State<WowImage> createState() => _WowImageState();
}

class _WowImageState extends State<WowImage> {
  String? thumbnailUrl;
  String? imageUrl;
  String pathImage = '', pathSmall = '';

  @override
  void didUpdateWidget(covariant WowImage oldWidget) {
    String url = widget._url.isNotEmpty
        ? widget._url
        : Methods.getString(widget.dataImage, pathImage);
    if (imageUrl != url) {
      setState(() {
        _loadImage();
      });
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void initState() {
    _loadImage();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return _boderImage(_cacheImage());
  }

  Widget _boderImage(Widget child){
    if(widget._borderRadius ==null){
      return child;
    }else{
      return ClipRRect(
        borderRadius: widget._borderRadius,
        child: child,
      );
    }
  }

// cache hình ảnh gốc
  CachedNetworkImage _cacheImage() {
    return CachedNetworkImage(
      imageUrl: Methods.getURL(imageUrl!),
      width: widget.width,
      height: widget.height,
      fit: widget.boxFit,
      errorWidget: (_, __, ___) {
        return widget.errorBuilder ?? errorImage();
      },
      progressIndicatorBuilder: (_, __, ___) {
        return widget.loaddingBuilder ?? Stack(
          children: [
            Opacity(
              opacity: 1.0,
              child: ClipRect(
                child: BackdropFilter(
                    filter: ImageFilter.blur(sigmaX: 20, sigmaY: 20),
                    child: Container(
                      color: Colors.black.withOpacity(0),
                      child: Opacity(
                          opacity: 0.0,
                          child: SizedBox(
                            width: widget.width ?? double.infinity,
                            height: widget.height ?? double.infinity,
                          )),
                    )),
              ),
            ),
            _imageNetWork(
              Methods.getURL(thumbnailUrl!),
            )
          ],
        );
      },
    );
  }

  Image _imageNetWork(
    String src, {
    Widget? errorBuider,
    Color? color,
    BlendMode? blendMode,
    Widget Function(BuildContext, Widget, int?, bool)? frameBuilder,
  }) {
    return Image(
      image: NetworkImage(src, scale: widget.scale),
      color: color,
      gaplessPlayback: true,
      excludeFromSemantics: true,
      colorBlendMode: blendMode,
      errorBuilder: ((context, error, stackTrace) =>
          errorBuider ?? errorImage()),
      width: widget.width ?? double.infinity,
      height: widget.height ?? double.infinity,
      fit: widget.boxFit,
      frameBuilder: frameBuilder ??
          ((context, child, frame, wasSynchronouslyLoaded) {
            if (frame != null) {
              return child;
            }
            return errorImage();
          }),
    );
  }

  Image errorImage({Color? color, BlendMode? blendMode}) {
    return Image.asset(
      'assets/images/default.jpg',
      width: widget.width ?? double.infinity,
      height: widget.height ?? double.infinity,
      fit: widget.boxFit,
    );
  }

// load hình ảnh
  void _loadImage() {
    if (pathSmall.isEmpty) {
      pathSmall = widget.fieldName != null
          ? widget.fieldName! + '_small_path'
          : 'hinhAnh_small_path';
    }
    if (pathImage.isEmpty) {
      pathImage = widget.fieldName != null
          ? widget.fieldName! + '_image_path'
          : 'hinhAnh_image_path';
    }

    thumbnailUrl = widget._url.isNotEmpty
        ? widget._url
        : Methods.getString(widget.dataImage, pathSmall);

    imageUrl = widget._url.isNotEmpty
        ? widget._url
        : Methods.getString(widget.dataImage, pathImage);
  }

}
